/*
 * Kerio IP filter
 * Copyright (C) Kerio Technologies s.r.o.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "hdrs.h"
#include "hooks.h"
#include "libdasm.h"

#if 0
static void
hexdump(
	u_int8_t *dp,
	int n
	)
{
	u_int8_t *start = NULL;
	u_int8_t *end = NULL;
	u_int8_t *p1 = NULL;
	u_int8_t *p2 = NULL;

	end = dp + n;	/* end is first location not to dump */

	start = dp;

	printk(KERN_INFO "------------------------------- raw data -------------------------------------\n");

	do {
		printk(KERN_INFO "%#lx:", (long) start);

		if (start >= end) {
			continue;
		}

		p2 = MIN(start + 16, end);

		for (p1 = start; p1 < p2; p1++) {

			printk(KERN_INFO " 0x%02X", (unsigned char) *p1);

			if (p1 == start + 7) {
				printk(KERN_INFO " ");
			}
		}

/*
		while (p1++ <= start + 16) {
			printk(KERN_INFO "   ");
		}

		for (p1 = start; p1 < p2; p1++) {
			printk(KERN_INFO "%c", *p1 > '\040' && *p1 < '\177' ? *p1 : '.');
		}
*/
		printk(KERN_INFO "\n");

	} while ((start += 16) < end);

	printk(KERN_INFO "--------------------------- end of raw data ----------------------------------\n");
}
#endif

static int
hook_get_bytes_to_replace(
	void *func,
	int min_bytes_to_replace
	)
{
	INSTRUCTION inst;
	int bytes_to_replace = 0;
	int ret = 0;
	//int i = 0;
	//int bytes = 8;

	while (bytes_to_replace < min_bytes_to_replace) {

		if (!(ret = get_instruction(&inst, func + bytes_to_replace, MODE_32))) {
			bytes_to_replace++;
			continue;
		}

		/*
		printf("%p  ", func + bytes_to_replace);

		if (bytes) {
			for (i = 0; i < MIN(bytes, ret); i++) {
				printf("%.2x", func[bytes_to_replace + i]);
			}
			printf("  ");
			for (i = MIN(bytes, ret); i < bytes*2 - ret; i++) {
				printf(" ");
			}
		}

		printf_instruction(&inst, FORMAT_INTEL);
		*/

		bytes_to_replace += ret;
	}

	return bytes_to_replace;
}

int
hook_prepare_func(
	void *func,
	int bytes_to_replace
	)
{
	unsigned int level = 0;
	pte_t *pte = NULL;
	int cnt = 0;
	if (!bytes_to_replace) {
		return -EINVAL;
	}
	while (cnt <= bytes_to_replace) {
		TRACE(LOG_INFO, "%s: getting page table for address %p+%d", __func__, func, cnt);
		if (!(pte = lookup_address((unsigned long)((unsigned char*)func+cnt), &level))) {
			TRACE(LOG_ERR, "%s: invalid page table.", __func__);
			return -EINVAL;
		}
		TRACE(LOG_INFO, "%s: page table %p is read-%s.", __func__, pte, ((pte_write(*pte) == _PAGE_RW) ? ("write") : ("only")));
		if (!pte_write(*pte)) {
			TRACE(LOG_INFO, "%s: changing to read-write", __func__);
			*pte = pte_mkwrite(*pte);
		}
		cnt += bytes_to_replace;
	};
	return 0;
}

int
hook_install(
	void *func,
	void *detour,
	void *trampoline
	)
{
	unsigned char jmpToOriginalFunction[5] = {0xE9, 0, 0, 0, 0};
	unsigned char jmpToDetourFunction[5] = {0xE9, 0, 0, 0, 0};
	int bytes_to_replace = 0;
	int ret;

	bytes_to_replace = hook_get_bytes_to_replace(func, sizeof(jmpToDetourFunction));

	if (bytes_to_replace < sizeof(jmpToDetourFunction) || bytes_to_replace > 15) {
		TRACE(LOG_ERR, "%s: invalid count of bytes to replace: %d", __func__, bytes_to_replace);
		return -EINVAL;
	}
	TRACE(LOG_INFO, "%s: total count of bytes to replace: %d", __func__, bytes_to_replace);

	if ((ret = hook_prepare_func(func, bytes_to_replace)) != 0) {
		return ret;
	}
	if ((ret = hook_prepare_func(trampoline, bytes_to_replace)) != 0) {
		return ret;
	}

	/* Create jmp to rest of function */
	*((unsigned int *)(&jmpToOriginalFunction[1])) =
		(unsigned int)(((unsigned char *)func + bytes_to_replace) - ((unsigned char *)trampoline + bytes_to_replace + sizeof(jmpToOriginalFunction)));

	*((unsigned int *)(&jmpToDetourFunction[1])) =
		(unsigned int)detour - ((unsigned int)func + sizeof(jmpToDetourFunction));

	/* Copy function prolog to trampoline */
	memcpy(trampoline, func, bytes_to_replace);

	/* Copy jmp to the rest of hooked routine to trampoline */
	memcpy((unsigned char*)trampoline + bytes_to_replace, jmpToOriginalFunction, sizeof(jmpToOriginalFunction));

	/* Clear first x bytes of hooked routine */
	memset(func, 0x90 /* NOP */, bytes_to_replace);

	/* Copy jmp to detour function to hooked routine */
	memcpy(func, jmpToDetourFunction, sizeof(jmpToDetourFunction));

	return bytes_to_replace;
}

int
hook_remove(
	void *func,
	void *trampoline,
	int bytes_to_remove
	)
{
	if (bytes_to_remove <= 0) {
		TRACE(LOG_WARN, "%s: nothing to remove.", __func__);
		return -EINVAL;
	}

	/* Restore origin function prologue from trampoline routine */
	memcpy(func, trampoline, bytes_to_remove);

	return 0;
}
