/*
 * Kerio IP filter
 * Copyright (C) Kerio Technologies s.r.o.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "hdrs.h"

void module_unload(void)
{
	TRACE(LOG_INFO, "%s: ********* unloading Kerio IP filter. *********", __func__);

	/* close /proc/linipf... */
	proc_close();
	/* close net device notifier */
	netdev_close();
	/* remove IO device */
	io_dev_remove();
	/* cleanup reasm engine */
	ip_frag_close();
	/* close IP filter - it MUST be closed after other components, because it will free queue lock. */
	ip_filter_close();
	/* close configuration */
	cfg_close();

	TRACE(LOG_INFO, "%s: ********* Kerio IP filter unloaded. *********", __func__);
}

int module_load(void)
{
	int err = 0;

	TRACE(LOG_INFO, "%s: ********* loading Kerio IP filter, API ver.: %d.%d *********", __func__, KIPF_MAJOR_VERSION, KIPF_MINOR_VERSION);

	do {
		/* load configuration */
		if ((err = cfg_init())) {
			break;
		}
		/* initialize IP filter - it MUST be first */
		if ((err = ip_filter_init())) {
			break;
		}
		/* initialize reasm engine */
		if ((err = ip_frag_init())) {
			break;
		}
		/* initialize and create IO device */
		if ((err = io_dev_init())) {
			break;
		}
		/* initialize net device notifier - it will fill main queue with net devices */
		if ((err = netdev_init())) {
			break;
		}
		/* initialize and create /proc/linipf */
		if ((err = proc_init())) {
			break;
		}
	} while (0);

	/* On error perform rollback action */
	if (err) {
		TRACE(LOG_INFO, "%s: performing rollback...", __func__);
		module_unload();
		TRACE(LOG_INFO, "%s: ********* Kerio IP filter was not loaded due to error %d *********", __func__, err);
		return err;
	}
	TRACE(LOG_INFO, "%s: ********* Kerio IP filter loaded. *********", __func__);
	return 0;
}

module_init(module_load)
module_exit(module_unload)

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Copyright (C) Kerio Technologies s.r.o.");
MODULE_DESCRIPTION("Kerio IP filter.");

