/*
 * Kerio IP filter
 * Copyright (C) Kerio Technologies s.r.o.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "hdrs.h"
#include "pkt.h"

int io_write_handle_packet(struct kipf_packet *packet)
{
	unsigned int magic;

	if (get_user(magic, (u_int32_t __user *) &packet->header.magic)) {
		TRACE(LOG_WARN, "%s: packet %p segmentation fault.", __func__, packet);
		return -EINVAL;
	}
	
	switch (magic) {
		case KIPF_ACTION_ACCEPT:
			return pkt_accept(packet);
		case KIPF_ACTION_DROP:
			return pkt_drop(packet);
		case KIPF_ACTION_COMPLETE:
			return pkt_complete(packet);
		default:
			TRACE(LOG_WARN, "%s: unimplemented action type %d", __func__, magic);
	}
	return -EINVAL;
}

ssize_t
io_write(
	struct file *file,
	const struct iovec *iv,
	ssize_t len
	)
{
	u_int32_t i = 0;
	ssize_t written = 0;
	struct kipf_request writeRequest;

	if (len != sizeof(struct kipf_request)) {
		TRACE(LOG_WARN, "%s: invalid length: %d", __func__, len);
		return -EINVAL;
	}

	if (memcpy_fromiovecend((unsigned char*) &writeRequest, iv, 0, len)) {
		TRACE(LOG_WARN, "%s: memcpy_fromiovec(): segmentation fault.", __func__);
		return -EFAULT;
	}

	if (writeRequest.count > MAX_RW_BUFFERS_COUNT) {
		TRACE(LOG_WARN, "%s: invalid count of requests (%d).", __func__, writeRequest.count);
		writeRequest.count = MAX_RW_BUFFERS_COUNT;
	}

	for (i = 0; i < writeRequest.count; i++) {
		int err;

		if ( ! writeRequest.packets[i]) {
			continue;
		}
		
		err = io_write_handle_packet((struct kipf_packet*) writeRequest.packets[i]);

		if (!err) {
			written++;
		}
	}

	writeRequest.count = written;

    return written;
}
