/*
 * Kerio IP filter
 * Copyright (C) Kerio Technologies s.r.o.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "hdrs.h"

/**
 *	Locals
 */
struct proc_dir_entry *gs_max_buffers_ent = NULL;
struct proc_dir_entry *gs_max_buffers_in_queue_ent = NULL;
struct proc_dir_entry *gs_debug_flag_ent = NULL;
struct proc_dir_entry *gs_status_ent = NULL;
struct proc_dir_entry *gs_new_dir_ent = NULL;

static ssize_t proc_read_status(struct file *file, char __user *buf, size_t count, loff_t *offset)
{
	if (count < 256) {
		TRACE(LOG_WARN, "%s: invalid len. (%d)", __func__, count);
		return -EINVAL;
	}
	if (count < 8) {
		TRACE(LOG_WARN, "%s: invalid len. (%d)", __func__, count);
		return -EINVAL;
	}
	return sprintf(buf, "%d\n", g_linipf_cfg.max_buffers);
	return sprintf(buf,
			"device: %s\n"
			"main queue len: %d\n"
			"pending queue len: %d\n",
			((g_device_opened) ? ("opened") : ("closed")),
			queue_len(&g_main_queue),
			queue_len(&g_pending_queue)
			);
}

static ssize_t proc_read_log_flags(struct file *file, char __user *buf, size_t count, loff_t *offset)
{
	if (count < 8) {
		TRACE(LOG_WARN, "%s: invalid len. (%d)", __func__, count);
		return -EINVAL;
	}
	return sprintf(buf, "%d\n", g_linipf_cfg.log_flags);
}

static ssize_t proc_write_log_flags(struct file *file, const char __user * buf, size_t count, loff_t *offset)
{
	char tmp[8];
	memset(&tmp, 0, sizeof(tmp));

	if (count > sizeof(tmp)) {
		TRACE(LOG_WARN, "%s: invalid len. (%d)", __func__, count);
		return -EINVAL;
	}
	if (copy_from_user(tmp, buf, count)) {
		TRACE(LOG_WARN, "%s: get_user(): segmentation fault.", __func__);
		return -EFAULT;
	}
	if (sscanf(tmp, "%d", &g_linipf_cfg.log_flags) != 1) {
		TRACE(LOG_WARN, "%s: sscanf(): no field assigned.", __func__);
		return -EINVAL;
	}
	TRACE(LOG_INFO, "%s: log_flags set to %d", __func__, g_linipf_cfg.log_flags);
	return count;
}

static ssize_t proc_read_max_buffers(struct file *file, char __user *buf, size_t count, loff_t *offset)
{
	if (count < 8) {
		TRACE(LOG_WARN, "%s: invalid len. (%d)", __func__, count);
		return -EINVAL;
	}
	return sprintf(buf, "%d\n", g_linipf_cfg.max_buffers);
}

static ssize_t proc_write_max_buffers(struct file *file, const char __user * buf, size_t count, loff_t *offset)
{
	char tmp[8];
	memset(&tmp, 0, sizeof(tmp));

	if (count > sizeof(tmp)) {
		TRACE(LOG_WARN, "%s: invalid len. (%d)", __func__, count);
		return -EINVAL;
	}
	if (copy_from_user(tmp, buf, count)) {
		TRACE(LOG_WARN, "%s: get_user(): segmentation fault.", __func__);
		return -EFAULT;
	}
	if (sscanf(tmp, "%d", &g_linipf_cfg.max_buffers) != 1) {
		TRACE(LOG_WARN, "%s: sscanf(): no field assigned.", __func__);
		return -EINVAL;
	}
	if (g_linipf_cfg.max_buffers < 65536) {
		TRACE(LOG_INFO, "%s: max_buffers can't be lower than 65536, setting to appropriate value.", __func__);
		g_linipf_cfg.max_buffers = 65536;
	}
	TRACE(LOG_INFO, "%s: max_buffers set to %d", __func__, g_linipf_cfg.max_buffers);
	return count;
}

static ssize_t proc_read_max_buffers_in_queue(struct file *file, char __user *buf, size_t count, loff_t *offset)
{
	if (count < 8) {
		TRACE(LOG_WARN, "%s: invalid len. (%d)", __func__, count);
		return -EINVAL;
	}
	return sprintf(buf, "%d\n", g_linipf_cfg.max_buffers_in_queue);
}

static ssize_t proc_write_max_buffers_in_queue(struct file *file, const char __user * buf, size_t count, loff_t *offset)
{
	char tmp[8];
	memset(&tmp, 0, sizeof(tmp));

	if (count > sizeof(tmp)) {
		TRACE(LOG_WARN, "%s: invalid len. (%d)", __func__, count);
		return -EINVAL;
	}
	if (copy_from_user(tmp, buf, count)) {
		TRACE(LOG_WARN, "%s: get_user(): segmentation fault.", __func__);
		return -EFAULT;
	}
	if (sscanf(tmp, "%d", &g_linipf_cfg.max_buffers_in_queue) != 1) {
		TRACE(LOG_WARN, "%s: sscanf(): no field assigned.", __func__);
		return -EINVAL;
	}
	if (g_linipf_cfg.max_buffers_in_queue < 1024) {
		TRACE(LOG_INFO, "%s: max_buffers_in_queue can't be lower than 1024, setting to appropriate value.", __func__);
		g_linipf_cfg.max_buffers_in_queue = 1024;
	}
	TRACE(LOG_INFO, "%s: max_buffers_in_queue set to %d", __func__, g_linipf_cfg.max_buffers_in_queue);
	return count;
}

static struct file_operations gs_debug_flag_operations = {
        .read = proc_read_log_flags,
        .write = proc_write_log_flags
};

static struct file_operations gs_status_operations = {
		.read = proc_read_status
};

static struct file_operations gs_max_buffers_operations = {
        .read = proc_read_max_buffers,
        .write = proc_write_max_buffers
};

static struct file_operations gs_max_buffers_in_queue_operations = {
        .read = proc_read_max_buffers_in_queue,
        .write = proc_write_max_buffers_in_queue
};

int32_t proc_init(void)
{
	if ( ! (gs_new_dir_ent = proc_mkdir("kipf", init_net.proc_net))) {
		TRACE(LOG_ERR, "%s: proc_mkdir(): can't create \"/proc/net/kipf\" directory.", __func__);
		return -1;
	}

	TRACE(LOG_INFO, "%s: directory \"/proc/net/kipf\" created.", __func__);

	if ( ! (gs_debug_flag_ent = proc_create("log_flags", S_IWUSR | S_IRUGO, gs_new_dir_ent, &gs_debug_flag_operations))) {
		TRACE(LOG_ERR, "%s: create_proc_entry(): can't create \"/proc/net/kipf/log_flags\" entry.", __func__);
		return -1;
	}

	TRACE(LOG_INFO, "%s: file \"/proc/net/kipf/log_flags\" created.", __func__);

	if ( ! (gs_status_ent = proc_create("status", S_IRUGO, gs_new_dir_ent, &gs_status_operations))) {
		TRACE(LOG_ERR, "%s: create_proc_entry(): can't create \"/proc/net/kipf/status\" entry.", __func__);
		return -1;
	}

	TRACE(LOG_INFO, "%s: file \"/proc/net/kipf/status\" created.", __func__);

	if ( ! (gs_max_buffers_ent = proc_create("max_buffers", S_IWUSR | S_IRUGO, gs_new_dir_ent, &gs_max_buffers_operations))) {
		TRACE(LOG_ERR, "%s: create_proc_entry(): can't create \"/proc/net/kipf/max_buffers\" entry.", __func__);
		return -1;
	}

	TRACE(LOG_INFO, "%s: file \"/proc/net/kipf/max_buffers\" created.", __func__);

	if ( ! (gs_max_buffers_in_queue_ent = proc_create("max_buffers_in_queue", S_IWUSR | S_IRUGO, gs_new_dir_ent, &gs_max_buffers_in_queue_operations))) {
		TRACE(LOG_ERR, "%s: create_proc_entry(): can't create \"/proc/net/kipf/max_buffers_in_queue\" entry.", __func__);
		return -1;
	}

	TRACE(LOG_INFO, "%s: file \"/proc/net/kipf/max_buffers_in_queue\" created.", __func__);

	return 0;
}

int32_t
proc_close(
	void
	)
{
	if (IS_PTR_VALID(gs_new_dir_ent)) {
		if (IS_PTR_VALID(gs_max_buffers_in_queue_ent)) {
			remove_proc_entry("max_buffers_in_queue", gs_new_dir_ent);
			TRACE(LOG_INFO, "%s: file \"/proc/net/kipf/max_buffers_in_queue\" removed.", __func__);
			gs_max_buffers_in_queue_ent = NULL;
		}
		if (IS_PTR_VALID(gs_max_buffers_ent)) {
			remove_proc_entry("max_buffers", gs_new_dir_ent);
			TRACE(LOG_INFO, "%s: file \"/proc/net/kipf/max_buffers\" removed.", __func__);
			gs_max_buffers_ent = NULL;
		}
		if (IS_PTR_VALID(gs_status_ent)) {
			remove_proc_entry("status", gs_new_dir_ent);
			TRACE(LOG_INFO, "%s: file \"/proc/net/kipf/status\" removed.", __func__);
			gs_status_ent = NULL;
		}
		if (IS_PTR_VALID(gs_debug_flag_ent)) {
			remove_proc_entry("log_flags", gs_new_dir_ent);
			TRACE(LOG_INFO, "%s: file \"/proc/net/kipf/log_flags\" removed.", __func__);
			gs_debug_flag_ent = NULL;
		}

		remove_proc_entry("kipf", init_net.proc_net);

		TRACE(LOG_INFO, "%s: directory \"/proc/net/kipf\" removed.", __func__);

		gs_new_dir_ent = NULL;
	}

	return 0;
}
