/*
 * Kerio IP filter
 * Copyright (C) Kerio Technologies s.r.o.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __QUEUE_H__
#define __QUEUE_H__

//
//	GET_CB - move it at the end
//
#define GET_CB(skb)	( (struct ipf_skb_cb*) ( ( (skb)->cb + sizeof((skb)->cb) ) - sizeof(struct ipf_skb_cb) ) )

#define NETWORK_PACKET	0x1
#define NET_DEV_PACKET	0x0

#define IPF_SKB_TAG	0x12345678

//
//	ipf_skb_cb - minimalize it's length to 8 bytes
//
struct ipf_skb_cb {
	u_int32_t tag;
	struct nf_queue_entry *nf_data;
	u_int32_t copied_len:16,
		  type:1,
		  received:1,
		  policy_routing:1;
	struct sk_buff *next;	// next IPv6 original fragment
	struct sk_buff *reasm;	// IPv6 reasembled packet
};

typedef int (*ipf_cmpfn)(struct sk_buff*, void*);

struct ipf_queue {
	struct sk_buff_head q;
	K_LOCK lock;
	int enabled;
	unsigned int queued_pkts;
};

void
queue_enable(
	struct ipf_queue *queue,
	int enable
	);

int
queue_cmp_by_id(
	struct sk_buff *skb,
	void *id
	);

int
queue_len(
	struct ipf_queue *queue
	);

int
queue_is_empty(
	struct ipf_queue *queue
	);

int
queue_insert(
	struct ipf_queue *queue,
	struct sk_buff *skb,
	int tail
	);

struct sk_buff*
queue_remove(
	struct ipf_queue *queue,
	void *data,
	ipf_cmpfn cmpfn
	);

void
queue_init(
	struct ipf_queue *queue
	);

void
queue_close(
	struct ipf_queue *queue
	);

#endif //__QUEUE_H__
