/*
 * Kerio IP filter
 * Copyright (C) Kerio Technologies s.r.o.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __UTILS_H__
#define __UTILS_H__

#define IP_ADDRESS_STRING(x) (((u_int8_t*)&x)[0])&255,(((u_int8_t*)&x)[1])&255,(((u_int8_t*)&x)[2])&255,(((u_int8_t*)&x)[3])&255

#if DBG
#define DbgBrk() asm volatile("int $0x3;")
#else
#define DbgBrk()
#endif	//	DBG

/**
 *	Log flags
 */
#define LOG_DBG		0x00000002
#define LOG_INFO	0x00000004
#define LOG_WARN	0x00000008
#define LOG_ERR		0x00000010
#define LOG_PKT		0x00000020
#define LOG_RAW		0x10000000

/*
 *	performance optimization, do not evaluate arguments if event disabled
 */
#define TRACE(flag, format, ...)							\
	if (!(flag & LOG_DBG) || (g_linipf_cfg.log_flags & 1))	\
		trace(flag, format, __VA_ARGS__);


#define TRACE_DBG(format, ...)							    \
	do { if (unlikely(g_linipf_cfg.log_flags & 1))          \
		printk(KERN_INFO format, __VA_ARGS__);              \
	} while (0)

#define TRACE_PKT(format, ...)							    \
	do { if (unlikely(g_linipf_cfg.log_flags))				\
		printk(KERN_INFO format, __VA_ARGS__);              \
	} while (0)

#ifndef IS_PTR_VALID
#define IS_PTR_VALID(x)	((x != NULL) ? (1) : (0))
#endif

#ifndef ARRAY_CNT
#define ARRAY_CNT(a) (sizeof(a)/sizeof((a)[0]))
#endif

#ifndef MIN
#define MIN(a, b)   (((a) < (b)) ? (a) : (b))
#endif

#ifndef MAX
#define MAX(a, b)   (((a) > (b)) ? (a) : (b))
#endif

typedef struct {
	spinlock_t spinlock;
} K_LOCK, *PK_LOCK;

int
alloc_lock(
	K_LOCK *lock
	);

void
free_lock(
	K_LOCK *lock
	);

void
acquire_lock(
	K_LOCK *lock
	);

void
release_lock(
	K_LOCK *lock
	);

void
trace(
	int flag,
	char *fmt,
	...
	);


void
dump_data(
	unsigned char *data,
	int len
	);

void
dump_eth_frame(
	unsigned char *data,
	int len
	);

void
dump_ip_packet(
	unsigned char *prefix,
	struct sk_buff *skb
	);

void
adjust_checksum(
	unsigned short *sum,
	void *odata,
	void *ndata,
	int len
	);

unsigned short
checksum(
	unsigned char *buff,
	unsigned int len
	);

void
dump_skb(
	struct sk_buff *skb
	);

void init_cb(struct sk_buff *skb, u_int8_t type);

#endif // __UTILS_H__
